#!/bin/sh

usage() {
    cat <<EOF
Usage:
    $0 [options] repo_old repo_new
        compares repositories by their URLs
    $0 [options] sources_old.list sources_new.list
        compares repositories by their sources.list files
Options:
    --components main[,contrib]         components to be compared
    --codename codename[,codename_new]  repository codename(s)
    --source-repo <source_repo_new>
        create changelog report, use <source_repo_new> for a changelog
    --project-keys ALPHA[,BETA]         jira project keys
    --merge-fixed <table>
        merge fixed issues to the changelog report
        the table format is: package version bugid
    --arch amd64        architectures to be compared
    --workdir WORKDIR   use WORKDIR instead of $workdir
    --profile           turn on profiling

Output options:
    --markup html           generate HTML report
    --skip-binary-diff      skip Packages.gz based diff
    --skip-source-diff      skip Sources.gz based diff
    --skip-binary-column    show source packages only
EOF
    exit 0
}

error() {
    >&2 echo "Error: $@"
    exit 1
}

set_codename() {
    local codename
    codename="$1"

    codename_old="$codename"
    codename_new="$codename"

    case "$codename" in
        *,*)
            codename_old=$(awk -F , '{ print $1 }')
            codename_new=$(awk -F , '{ print $2 }')
            ;;
    esac
}

parse_options() {
    set -eo pipefail

    tab=$(echo -e '\t')

    components=main,contrib,non-free
    arch=$(dpkg --print-architecture)

    repo_old=
    repo_new=
    codename1=
    codename2=
    skip_binary_diff=
    skip_source_diff=
    skip_binary_column=
    repo_src=
    report_markup=text
    project_keys="--project-keys BT,DEVOS"
    workdir="/tmp/$USER/repdiff"

    while test -n "$1"
    do
        case "$1" in
            --codename)
                set_codename "$2"
                shift 2
                ;;
            -c|--components)
                components="$2"
                shift 2
                ;;
            -a|--arch)
                arch="$2"
                shift 2
                ;;
            --profile)
                profile_python="-m cProfile -s time"
                shift
                ;;
            --project-keys)
                project-keys="--project-keys $2"
                shift 2
                ;;
            --markup)
                report_markup="$2"
                shift 2
                ;;
            --skip-binary-diff)
                skip_package_diff="--skip-binary-diff"
                shift
                ;;
            --skip-source-diff)
                skip_package_diff="--skip-source-diff"
                shift
                ;;
            --skip-binary-column)
                skip_binary_column="--skip-binary-column"
                shift
                ;;
            --source-repo)
                repo_src="$2"
                source_repo="--create-changelog --source-repo $repo_src"
                shift 2
                ;;
            --merge-fixed)
                fixed_table="$2"
                test -f "$fixed_table" || error "Cannot find $fixed_table"
                shift 2
                ;;
            --workdir)
                workdir="$2"
                shift 2
                ;;
            --help|-h)
                usage
                exit 0
                ;;
            *)
                url="$1"
                test ! -f "$url" || url=$(realpath "$url")

                if test -z "$repo_old"
                then
                    repo_old="$url"
                else
                    repo_new="$url"
                fi
                shift
                ;;
        esac
    done

    components=$(echo "$components" | tr , ' ')
    mkdir -p "$workdir"

    set +vx
    . "$script_dir/$report_markup".sh
    set -$opts
    cat "$script_dir"/prefix."$report_markup"
}

create_cache() {
    mkdir -p "$workdir"/old "$workdir"/new

    (
        cd "$workdir"/old
        repo_create_source_list "$repo_old" "$codename_old"
    )

    test -z "$repo_new" || (
        cd "$workdir"/new
        repo_create_source_list "$repo_new" "$codename_new"
    )
}

create_package_diff() (
    test -n "$repo_new" || return 0

    cd "$workdir"
    python3 $profile_python "$script_dir"/repdiff.py --markup "$report_markup" --arch "$arch" $skip_binary_diff $skip_source_diff $source_repo $skip_binary_column $project_keys --workdir "$workdir" old new
)


