#!/usr/bin/python3

import sys, argparse, re, os, subprocess
from markup import Markup
from repos import get_cache
from apt_pkg import version_compare


def parse_args():
    par = argparse.ArgumentParser(description = 'Create a cache diff between two repositories')
    par.add_argument('old', action='store', help='first sources.list dir')
    par.add_argument('new', action='store', help='second sources.list dir')
    par.add_argument('--markup', dest='markup', default='text', action='store', help='html or text markup')
    par.add_argument('--arch', dest='arch', default='amd64', action='store', help='binary architecture to process')
    par.add_argument('--bug-re', dest='bug_re', action='store', default='[A-Z][A-Z]\+-\d\d+', help='a regular expression to extract bugs')
    par.add_argument('--create-changelog', dest='create_changelog', action='store_true', help='create a changelog report')
    par.add_argument('--project-keys', dest='project_keys', action='store', help='jira project keys to extract')
    par.add_argument('--skip-binary-diff', dest='skip_binary_diff', action='store_true', help='do not print binary differences')
    par.add_argument('--skip-source-diff', dest='skip_source_diff', action='store_true', help='do not print source differences')
    par.add_argument('--source-repo', dest='source_repo', action='store', help='extract a changelog from a source repo')
    par.add_argument('--workdir', dest='workdir', action='store', help='use WORKDIR for temporary files')
    par.add_argument('--skip-binary-column', dest='skip_binary_column', action='store_true', help='skip binary packages in description table')

    args = par.parse_args()
    if args.project_keys:
        args.bug_re = '|'.join(x + '-\d+' for x in args.project_keys.split(','))
    args.bug_re = re.compile(args.bug_re) 

    if not 'get_changes_script' in args:
        args.get_changes_script = os.path.join(os.path.dirname(sys.argv[0]), 'get-changes.sh')
    return args


def add_cache_diff(name, old, new, markup):
    added = []
    removed = []
    upgraded = []
    downgraded = []
    rebuilt = []

    for k in new.keys():
        v_new = new[k]['Version']
        if not k in old:
            added.append([k, v_new])
            continue

        if old[k].md5 == new[k].md5: continue

        v_old = old[k]['Version']
        c = version_compare(v_old, v_new)
        if c < 0:
            upgraded.append([k, v_old, v_new])
            continue
        if c > 0:
            downgraded.append([k, v_old, v_new])
            continue

        rebuilt.append([k, v_old])

    for k in old.keys():
        if not k in new:
            removed.append([k, old[k]['Version']])

    markup.add_table('Summary for ' + name, ['Added', 'Removed', 'Upgraded', 'Downgraded', 'Rebuilt'],
        [[str(len(x)) for x in [added, removed, upgraded, downgraded, rebuilt]]])
    markup.add_table('Added ' + name, ['Package', 'Version'], added)
    markup.add_table('Removed ' + name, ['Package', 'Version'], removed)
    markup.add_table('Upgraded ' + name, ['Package', 'Old', 'New'], upgraded)
    markup.add_table('Downgraded ' + name, ['Package', 'Old', 'New'], downgraded)
    markup.add_table('Rebuilt ' + name, ['Package', 'Version'], rebuilt)


def get_changes(package, old_version, args):
    a = ['sh', args.get_changes_script, package, old_version, args.source_repo, args.workdir]
    output = subprocess.check_output(a).decode('utf-8').strip()
    return output


def get_cve(changes):
    l = re.findall(r'CVE-\d+-\d+', changes)
    l = list(set(l))
    l.sort()
    return l


def get_bug(changes, bug_re):
    l = re.findall(bug_re, changes)
    l = list(set(l))
    l.sort()
    return l
    

def create_changelog(old, new, args):
    added = []
    added_src_ver = {}
    added_src_pkg = {}
    upgraded_src = {}

    for k in new.keys():
        v_new = new[k]['Version']
        s_new = new[k]['Source'] if 'Source' in new[k] else k

        if not k in old:
            added_src_ver[s_new] = v_new
                
            r = added_src_pkg.get(s_new, [])
            r.append(k)
            added_src_pkg[s_new] = r
            continue

        if old[k].md5 == new[k].md5: continue

        v_old = old[k]['Version']
        c = version_compare(v_old, v_new)
        if c < 0:
            upgraded_src[s_new] = (v_old, v_new)
            continue

    for k in sorted(added_src_ver.keys()):
        first = True
        for p in sorted(added_src_pkg[k]):
            d = new[p]['Description'].split('\n', maxsplit = 1)
            d = d[0]
            if first:
                row = [k, added_src_ver[k], p, d]
            else:
                row = ['', '', p, d]
                first = False
            added.append(row)

    bug_list = []
    cve_list = []
    change_list = []

    for k in sorted(upgraded_src.keys()):
        old_version = upgraded_src[k][0]
        version = upgraded_src[k][1]
        change = get_changes(k, old_version, args)

        change_list.append([k, version, change])

        cve = ' '.join(get_cve(change))
        bug = ' '.join(get_bug(change, args.bug_re))

        if cve:
            cve_list.append([k, version, cve, bug])
        elif bug:
            bug_list.append([k, version, bug])

    return added, cve_list, bug_list, change_list


def prepare_sections(args):
    markup = Markup.get(args.markup)

    old_src, old_bin, old_release = get_cache(args.old, args.old, args.arch)
    new_src, new_bin, new_release  = get_cache(args.new, args.new, args.arch)

    old_codename = old_release['Codename']
    new_codename = new_release['Codename']
    new_codename = new_codename + ' ' if old_codename != new_codename else ''
    markup.add_h1('Diff ' + old_codename + ' ' + old_release['Version'] + ' vs ' + new_codename + new_release['Version'])

    if args.create_changelog:
        added, cve, bug, change = create_changelog(old_bin, new_bin, args)
        if args.skip_binary_column:
            markup.add_table('Added', ['Source', 'Version', 'Description'],
                [[r[0], r[1], r[3]] for r in added])
        else:
            markup.add_table('Added', ['Source', 'Version', 'Package', 'Description'], added)

        markup.add_table('Fixed CVE', ['Package', 'Version', 'CVE', 'Bugfixes'], cve)
        markup.add_table('Fixed bugs', ['Package', 'Version', 'Bugfixes'], bug)
        markup.add_table('Changelog', ['Package', 'Version', 'Changes'], change)

    if not args.skip_source_diff: add_cache_diff('sources', old_src, new_src, markup)
    if not args.skip_binary_diff: add_cache_diff('binaries', old_bin, new_bin, markup)

    return markup


ARGS = parse_args()

markup = prepare_sections(ARGS)
markup.print()

