#!/usr/bin/python3

import sys, os.path, argparse, gzip

import pycurl
from io import BytesIO

from apt_pkg import uri_to_filename, version_compare
from aptsources.sourceslist import SourceEntry
from debian_bundle import deb822


def read_curl(cache_dir, url):
    path = get_path_curl(cache_dir, url)
    with open(path) as f: return f.read()


def zcat_curl(cache_dir, url):
    path = get_path_curl(cache_dir, url)
    with gzip.open(path, 'r') as f: return f.read()


def overwrite_cache(bytesio, cache_filename):
    with open(cache_filename, 'wb') as f: f.write(bytesio.getbuffer())


def get_path_curl(cache_dir, url):
    FILE0 = 'file:'
    FILE2 = 'file://'
    FILE3 = 'file:///'

    if url.startswith(FILE0):
        if url.startswith(FILE3): return url[len(FILE2):]
        if url.startswith(FILE2): sys.exit('Invalid url: ' + url)
        return url[len(FILE0):]

    CURL = pycurl.Curl()
    # CURL.setopt(CURL.VERBOSE, 1) # debug
    CURL.setopt(CURL.URL, url)

    filename = os.path.join(cache_dir, uri_to_filename(url))

    if os.path.isfile(filename):
        m_time = os.path.getmtime(filename)
        CURL.setopt(CURL.TIMEVALUE, int(m_time))
        CURL.setopt(CURL.TIMECONDITION, CURL.TIMECONDITION_IFMODSINCE)

    b = BytesIO()
    CURL.setopt(CURL.WRITEDATA, b)
    CURL.perform()
    code = CURL.getinfo(CURL.RESPONSE_CODE)
    CURL.close()

    if code != 213 and code != 304:
        # file modified
        overwrite_cache(b, filename)

    return filename


def get_dists_uri(entry, component, archpath):
    return entry.uri + '/dists/' + entry.dist + '/' + component + '/' + archpath


def update_cache(cache, p, entry, component, md5):
    name = p['Package']
    p.entry = entry
    p.entry_component = component
    p.md5 = md5

    if not name in cache:
        cache[name] = p
        return

    version = p['Version']
    old_version = cache[name]['Version']
    if version_compare(old_version, version) < 0: cache[name] = p


def download_bin_cache(entry, cache, cache_dir, arch):
    archs = entry.architectures
    if not archs: archs = [arch]
    cache_paths = ['binary-' + a for a in archs]

    for path in cache_paths:
        for c in entry.comps:
            cfc = zcat_curl(cache_dir, get_dists_uri(entry, c, path) + "/Packages.gz")
            for par in deb822.Deb822.iter_paragraphs(cfc):
                p = deb822.Packages(par)
                update_cache(cache, p, entry, c, p['MD5Sum'])


def download_src_cache(entry, cache, cache_dir, arch):
    cache_name = 'Sources'

    for c in entry.comps:
        cfc = zcat_curl(cache_dir, get_dists_uri(entry, c, 'source') + "/Sources.gz")
        for par in deb822.Deb822.iter_paragraphs(cfc):
            p = deb822.Sources(par)
            md5 = p['Files']
            md5 = ''.join([l['MD5sum'] for l in md5])
            update_cache(cache, p, entry, 'source', md5)


def get_cache(sources_list_dir, cache_dir, arch):
    sources_list_path = os.path.abspath(os.path.join(sources_list_dir, 'sources.list'))

    src_cache = {}
    bin_cache = {}
    first_entry = True

    with open(sources_list_path) as f:
        for line in f:
            entry = SourceEntry(line)
            if entry.disabled: continue

            if first_entry:
                rfc = read_curl(cache_dir, entry.uri + "/dists/" + entry.dist + "/Release")
            
                release = deb822.Release(rfc)
                first_entry = False

            if entry.type == 'deb':
                download_bin_cache(entry, bin_cache, cache_dir, arch)
            else:
                download_src_cache(entry, src_cache, cache_dir, arch)

    return src_cache, bin_cache, release


